# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging
import pprint

from testinfra.backend import base
from testinfra.utils.ansible_runner import AnsibleRunner

logger = logging.getLogger("testinfra")


class AnsibleBackend(base.BaseBackend):
    NAME = "ansible"
    HAS_RUN_ANSIBLE = True

    def __init__(
        self,
        host,
        ansible_inventory=None,
        ssh_config=None,
        ssh_identity_file=None,
        force_ansible=False,
        *args,
        **kwargs,
    ):
        self.host = host
        self.ansible_inventory = ansible_inventory
        self.ssh_config = ssh_config
        self.ssh_identity_file = ssh_identity_file
        self.force_ansible = force_ansible
        super().__init__(host, *args, **kwargs)

    @property
    def ansible_runner(self):
        return AnsibleRunner.get_runner(self.ansible_inventory)

    def run(self, command, *args, **kwargs):
        command = self.get_command(command, *args)
        if not self.force_ansible:
            host = self.ansible_runner.get_host(
                self.host,
                ssh_config=self.ssh_config,
                ssh_identity_file=self.ssh_identity_file,
            )
            if host is not None:
                return host.run(command)
        out = self.run_ansible("shell", module_args=command, check=False)
        return self.result(
            out["rc"],
            command,
            stdout_bytes=None,
            stderr_bytes=None,
            stdout=out["stdout"],
            stderr=out["stderr"],
        )

    def run_ansible(self, module_name, module_args=None, **kwargs):
        def get_encoding():
            return self.encoding

        result = self.ansible_runner.run_module(
            self.host, module_name, module_args, get_encoding=get_encoding, **kwargs
        )
        logger.info(
            "RUN Ansible(%s, %s, %s): %s",
            repr(module_name),
            repr(module_args),
            repr(kwargs),
            pprint.pformat(result),
        )
        return result

    def get_variables(self):
        return self.ansible_runner.get_variables(self.host)

    @classmethod
    def get_hosts(cls, host, **kwargs):
        inventory = kwargs.get("ansible_inventory")
        return AnsibleRunner.get_runner(inventory).get_hosts(host or "all")



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

try:
    import salt.client
except ImportError:
    raise RuntimeError("You must install salt package to use the salt backend")

from testinfra.backend import base


class SaltBackend(base.BaseBackend):
    HAS_RUN_SALT = True
    NAME = "salt"

    def __init__(self, host, *args, **kwargs):
        self.host = host
        self._client = None
        super().__init__(self.host, *args, **kwargs)

    @property
    def client(self):
        if self._client is None:
            self._client = salt.client.LocalClient()
        return self._client

    def run(self, command, *args, **kwargs):
        command = self.get_command(command, *args)
        out = self.run_salt("cmd.run_all", [command])
        return self.result(
            out["retcode"],
            command,
            out["stdout"].encode("utf8"),
            out["stderr"].encode("utf8"),
            stdout=out["stdout"],
            stderr=out["stderr"],
        )

    def run_salt(self, func, args=None):
        out = self.client.cmd(self.host, func, args or [])
        if self.host not in out:
            raise RuntimeError(
                "Error while running {}({}): {}. "
                "Minion not connected ?".format(func, args, out)
            )
        return out[self.host]

    @classmethod
    def get_hosts(cls, host, **kwargs):
        if host is None:
            host = "*"
        if any(c in host for c in "@*[?"):
            client = salt.client.LocalClient()
            if "@" in host:
                hosts = client.cmd(host, "test.true", tgt_type="compound").keys()
            else:
                hosts = client.cmd(host, "test.true").keys()
            if not hosts:
                raise RuntimeError("No host matching '{}'".format(host))
            return sorted(hosts)
        return super().get_hosts(host, **kwargs)
